/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogsubversionadd.h"
#include "help.h"
#include "generalconfig.h"
#include "projectutils.h"
#include "subversion.h"
#include "progresswindow.h"


SubversionAddDialog::SubversionAddDialog (int dummy)
{
  subversionadddialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (subversionadddialog), "Add task");
  gtk_window_set_position (GTK_WINDOW (subversionadddialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (subversionadddialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (subversionadddialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  hbox1 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);

  label_reinitialize_books = gtk_label_new ("Store whole project to ensure that everything is in");
  gtk_widget_show (label_reinitialize_books);
  gtk_box_pack_start (GTK_BOX (hbox1), label_reinitialize_books, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label_reinitialize_books), 0, 0.5);

  /*
  button_select_books = gtk_button_new ();
  gtk_widget_show (button_select_books);
  gtk_box_pack_start (GTK_BOX (hbox1), button_select_books, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (button_select_books), alignment1);

  hbox2 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

  image1 = gtk_image_new_from_stock ("gtk-properties", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

  label2 = gtk_label_new_with_mnemonic ("_Select books");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox2), label2, FALSE, FALSE, 0);
  */

  dialog_action_area1 = GTK_DIALOG (subversionadddialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  help_button (dialog_action_area1, NULL, NULL);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (subversionadddialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (subversionadddialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  // g_signal_connect ((gpointer) button_select_books, "clicked", G_CALLBACK (on_button_select_books_clicked), gpointer(this));
  g_signal_connect ((gpointer) okbutton, "clicked", G_CALLBACK (on_okbutton_clicked), gpointer(this));

  gtk_widget_grab_focus (button_select_books);
  gtk_widget_grab_default (okbutton);
}


SubversionAddDialog::~SubversionAddDialog ()
{
  gtk_widget_destroy (subversionadddialog);
}


int SubversionAddDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (subversionadddialog));
}


void SubversionAddDialog::on_button_select_books_clicked (GtkButton *button, gpointer user_data)
{
  ((SubversionAddDialog *) user_data)->on_button_select_books ();
}


void SubversionAddDialog::on_button_select_books ()
{
}


void SubversionAddDialog::on_okbutton_clicked (GtkButton *button, gpointer user_data)
{
  ((SubversionAddDialog *) user_data)->on_okbutton ();
}


void SubversionAddDialog::on_okbutton ()
{
  /*
  Storing the whole project should not delete and then re-initialize the books, 
  because that takes unnessary space in the respository. 
  Instead, it should store all chapters in the books.
  This keeps the repository as small as possible.
  */
  GeneralConfiguration genconfig (0);
  ustring project = genconfig.project();
  vector<unsigned int> books = project_get_books (project);
  ProgressWindow progresswindow ("", false);
  progresswindow.set_iterate (0, 1, books.size());
  for (unsigned int bk = 0; bk < books.size(); bk++) {
    progresswindow.iterate ();
    vector <unsigned int> chapters = project_get_chapters (project, books[bk]);
    for (unsigned int ch = 0; ch < chapters.size(); ch++) {
      subversion_store_chapter (project, books[bk], chapters[ch]);
    }
  }
}
