/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogsubversionsetup.h"
#include "utilities.h"
#include "java.h"
#include "gtkwrappers.h"
#include "gwrappers.h"
#include "unixwrappers.h"
#include "directories.h"
#include "shell.h"
#include "projectconfig.h"
#include "gui.h"
#include "progresswindow.h"
#include "help.h"
#include "shortcuts.h"
#include "subversion.h"
#include "projectutils.h"
#include "books.h"


SubversionSetupDialog::SubversionSetupDialog (int dummy)
{
  // Config to set values.
  GeneralConfiguration genconfig (0);
  ProjectConfiguration projectconfig (genconfig.project());
  updatefrequency = genconfig.subversion_update_frequency ();

  Shortcuts shortcuts (0);
  
  // Build GUI.
  subversionsetupdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (subversionsetupdialog), "Subversion Repository Setup");
  gtk_window_set_position (GTK_WINDOW (subversionsetupdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (subversionsetupdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (subversionsetupdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 5);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  table1 = gtk_table_new (9, 6, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table1), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table1), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table1), 4);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_table_attach (GTK_TABLE (table1), hseparator1, 0, 6, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  vbox2 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox2);
  gtk_table_attach (GTK_TABLE (table1), vbox2, 0, 6, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox5);
  gtk_box_pack_start (GTK_BOX (vbox2), vbox5, FALSE, FALSE, 0);

  GSList *radiobutton_internal_group = NULL;
  
  radiobutton_internal = gtk_radio_button_new_with_mnemonic (NULL, "Automatic internal repository");
  gtk_widget_show (radiobutton_internal);
  gtk_box_pack_start (GTK_BOX (vbox5), radiobutton_internal, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_internal), radiobutton_internal_group);
  radiobutton_internal_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_internal));

  shortcuts.button (radiobutton_internal);
  
  radiobutton_external = gtk_radio_button_new_with_mnemonic (NULL, "Remote external repository");
  gtk_widget_show (radiobutton_external);
  gtk_box_pack_start (GTK_BOX (vbox5), radiobutton_external, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_external), radiobutton_internal_group);
  radiobutton_internal_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_external));

  shortcuts.button (radiobutton_external);

  hbox17 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox17);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox17, TRUE, TRUE, 0);

  table2 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (hbox17), table2, TRUE, TRUE, 0);
  gtk_table_set_row_spacings (GTK_TABLE (table2), 4);
  gtk_table_set_col_spacings (GTK_TABLE (table2), 4);

  labelpassword = gtk_label_new ("Password");
  gtk_widget_show (labelpassword);
  gtk_table_attach (GTK_TABLE (table2), labelpassword, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (labelpassword), 0, 0.5);
  
  shortcuts.label (labelpassword);

  labelusername = gtk_label_new ("Username");
  gtk_widget_show (labelusername);
  gtk_table_attach (GTK_TABLE (table2), labelusername, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (labelusername), 0, 0.5);

  shortcuts.label (labelusername);

  entry_password = gtk_entry_new ();
  gtk_widget_show (entry_password);
  gtk_table_attach (GTK_TABLE (table2), entry_password, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_entry_set_text (GTK_ENTRY (entry_password), projectconfig.repository_password().c_str());

  entry_username = gtk_entry_new ();
  gtk_widget_show (entry_username);
  gtk_table_attach (GTK_TABLE (table2), entry_username, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  gtk_entry_set_text (GTK_ENTRY (entry_username), projectconfig.repository_username().c_str());

  label_path = gtk_label_new ("Repository path");
  gtk_widget_show (label_path);
  gtk_table_attach (GTK_TABLE (table2), label_path, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label_path), 0, 0.5);

  shortcuts.label (label_path);

  entry_path = gtk_entry_new ();
  gtk_widget_show (entry_path);
  gtk_table_attach (GTK_TABLE (table2), entry_path, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox3);
  gtk_table_attach (GTK_TABLE (table1), vbox3, 4, 5, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  GSList *radiobutton1_group = NULL;

  radiobutton2 = gtk_radio_button_new_with_mnemonic (NULL, "Overwrite remote repository with local data");
  gtk_widget_show (radiobutton2);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton2, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton2), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton2));

  shortcuts.button (radiobutton2);

  radiobutton3 = gtk_radio_button_new_with_mnemonic (NULL, "Overwrite local data with remote repository");
  gtk_widget_show (radiobutton3);
  gtk_box_pack_start (GTK_BOX (vbox3), radiobutton3, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton3), radiobutton1_group);
  radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton3));

  shortcuts.button (radiobutton3);

  hseparator2 = gtk_hseparator_new ();
  gtk_widget_show (hseparator2);
  gtk_table_attach (GTK_TABLE (table1), hseparator2, 0, 6, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  label_repository_accessible_output = gtk_label_new ("Repository test output");
  gtk_widget_show (label_repository_accessible_output);
  gtk_table_attach (GTK_TABLE (table1), label_repository_accessible_output, 0, 6, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label_repository_accessible_output), 0, 0.5);

  image_repository_accessible_ok = gtk_image_new_from_stock ("gtk-apply", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image_repository_accessible_ok);
  gtk_table_attach (GTK_TABLE (table1), image_repository_accessible_ok, 0, 1, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label_repository_accessible_done = gtk_label_new ("Done");
  gtk_widget_show (label_repository_accessible_done);
  gtk_table_attach (GTK_TABLE (table1), label_repository_accessible_done, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label_repository_accessible_done), 0, 0.5);

  label_repository_accessible_info = gtk_label_new ("Repository info");
  gtk_widget_show (label_repository_accessible_info);
  gtk_table_attach (GTK_TABLE (table1), label_repository_accessible_info, 2, 6, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label_repository_accessible_info), 0, 0.5);

  image_tasks_running_ok = gtk_image_new_from_stock ("gtk-apply", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image_tasks_running_ok);
  gtk_table_attach (GTK_TABLE (table1), image_tasks_running_ok, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  label_tasks_running_ok = gtk_label_new ("Done");
  gtk_widget_show (label_tasks_running_ok);
  gtk_table_attach (GTK_TABLE (table1), label_tasks_running_ok, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label_tasks_running_ok), 0, 0.5);

  label_tasks_running_info = gtk_label_new ("Tasks running information");
  gtk_widget_show (label_tasks_running_info);
  gtk_table_attach (GTK_TABLE (table1), label_tasks_running_info, 2, 6, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label_tasks_running_info), 0, 0.5);

  hseparator3 = gtk_hseparator_new ();
  gtk_widget_show (hseparator3);
  gtk_table_attach (GTK_TABLE (table1), hseparator3, 0, 6, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  hseparator4 = gtk_hseparator_new ();
  gtk_widget_show (hseparator4);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator4, TRUE, TRUE, 0);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox4);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox4, TRUE, TRUE, 0);

  label53 = gtk_label_new ("If there is a conflict between the data on the server and Bibledit's data");
  gtk_widget_show (label53);
  gtk_box_pack_start (GTK_BOX (vbox4), label53, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label53), 0, 0.5);

  GSList *radiobutton_conflict_me_group = NULL;
  
  radiobutton_conflict_me = gtk_radio_button_new_with_mnemonic (NULL, "take my data");
  gtk_widget_show (radiobutton_conflict_me);
  gtk_box_pack_start (GTK_BOX (vbox4), radiobutton_conflict_me, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_conflict_me), radiobutton_conflict_me_group);
  radiobutton_conflict_me_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_conflict_me));
  
  shortcuts.button (radiobutton_conflict_me);

  radiobutton_conflict_server = gtk_radio_button_new_with_mnemonic (NULL, "take the server's data");
  gtk_widget_show (radiobutton_conflict_server);
  gtk_box_pack_start (GTK_BOX (vbox4), radiobutton_conflict_server, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_conflict_server), radiobutton_conflict_me_group);
  radiobutton_conflict_me_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_conflict_server));

  shortcuts.button (radiobutton_conflict_server);

  /*
  radiobutton_conflict_prompt = gtk_radio_button_new_with_mnemonic (NULL, "prompt what to do");
  gtk_widget_show (radiobutton_conflict_prompt);
  gtk_box_pack_start (GTK_BOX (vbox4), radiobutton_conflict_prompt, FALSE, FALSE, 0);
  gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton_conflict_prompt), radiobutton_conflict_me_group);
  radiobutton_conflict_me_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton_conflict_prompt));

  shortcuts.button (radiobutton_conflict_prompt);
  */
  
  // Set conflict handling values.
  SubversionConflictHandlingType conflicthandling = (SubversionConflictHandlingType) projectconfig.repository_conflict ();
  switch (conflicthandling) {
    case schtTakeMe:
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_conflict_me), true);
      break;
    }
    case schtTakeServer:
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_conflict_server), true);
      break;
    }
    /*
    case schtPrompt:
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_conflict_prompt), true);
      break;
    }
    */
  }
  
  hseparator5 = gtk_hseparator_new ();
  gtk_widget_show (hseparator5);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hseparator5, TRUE, TRUE, 0);

  hbox19 = gtk_hbox_new (FALSE, 5);
  gtk_widget_show (hbox19);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox19, TRUE, TRUE, 0);

  label54 = gtk_label_new ("Check for updates on the server every");
  gtk_widget_show (label54);
  gtk_box_pack_start (GTK_BOX (hbox19), label54, FALSE, FALSE, 0);
  
  shortcuts.label (label54);

  spinbutton_update_adj = gtk_adjustment_new (updatefrequency, 5, 3600, 1, 10, 10);
  spinbutton_update = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton_update_adj), 1, 0);
  gtk_widget_show (spinbutton_update);
  gtk_box_pack_start (GTK_BOX (hbox19), spinbutton_update, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton_update), TRUE);

  label55 = gtk_label_new ("seconds");
  gtk_widget_show (label55);
  gtk_box_pack_start (GTK_BOX (hbox19), label55, FALSE, FALSE, 0);

  dialog_action_area1 = GTK_DIALOG (subversionsetupdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  help_button (dialog_action_area1, &shortcuts, "subversion_setup");
  
  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (subversionsetupdialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  shortcuts.stockbutton (cancelbutton);
  
  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (subversionsetupdialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  shortcuts.stockbutton (okbutton);
  
  shortcuts.process ();

  g_signal_connect ((gpointer) radiobutton_internal, "toggled",
                    G_CALLBACK (on_radiobutton_internal_external_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) radiobutton_external, "toggled",
                    G_CALLBACK (on_radiobutton_internal_external_toggled),
                    gpointer(this));
  g_signal_connect ((gpointer) entry_password, "changed",
                    G_CALLBACK (on_entry_changed),
                    gpointer(this));
  g_signal_connect ((gpointer) entry_username, "changed",
                    G_CALLBACK (on_entry_changed),
                    gpointer(this));
  g_signal_connect ((gpointer) entry_path, "changed",
                    G_CALLBACK (on_entry_changed),
                    gpointer(this));
  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (on_okbutton_clicked),
                    gpointer(this));

  gtk_widget_grab_default (okbutton);
  gtk_label_set_mnemonic_widget (GTK_LABEL (label_path), entry_path);
  gtk_label_set_mnemonic_widget (GTK_LABEL (labelusername), entry_username);
  gtk_label_set_mnemonic_widget (GTK_LABEL (labelpassword), entry_password);
  gtk_label_set_mnemonic_widget (GTK_LABEL (label54), spinbutton_update);

  // Initialize object's variables.
  event_id_gui = 0;
  project_tasks_count = 1;
  previous_project_tasks_count = 0;
  projectchanged = false;

  // Set values for repository.
  // Extract the URL of the repository from the working copy.
  // If this URL is the local one, set the system to "internal repository",
  // else set it to "external repository" and set that URL in the entry.
  ustring project = genconfig.project();
  ustring url = subversion_project_get_url (project);
  if (url == subversion_repository_local (project, true)) {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_internal), true);
  } else {
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobutton_external), true);
    gtk_entry_set_text (GTK_ENTRY (entry_path), url.c_str());
  }

  // Gui update.
  event_id_tasks_ready = g_timeout_add_full (G_PRIORITY_DEFAULT, 600, GSourceFunc (tasks_counter_timeout), gpointer(this), NULL);
  tasks_counter ();
  set_gui ();
}


SubversionSetupDialog::~SubversionSetupDialog ()
{
  // Resume Subversion tasks.
  subversion_command_pause (false);

  // Destroy event sources.
  if (event_id_tasks_ready) {
    GSource *source = g_main_context_find_source_by_id (NULL, event_id_tasks_ready);
    if (source) g_source_destroy (source);
  }
  if (event_id_gui) {
    GSource *source = g_main_context_find_source_by_id (NULL, event_id_gui);
    if (source) g_source_destroy (source);
  }
  
  // Destroy widget.
  gtk_widget_destroy (subversionsetupdialog);
}


int SubversionSetupDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (subversionsetupdialog));
}


void SubversionSetupDialog::on_radiobutton_internal_external_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
  ((SubversionSetupDialog *) user_data)->on_radiobutton_internal_external ();
}


void SubversionSetupDialog::on_radiobutton_internal_external ()
{
  set_gui ();
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_external)))
    gtk_widget_grab_focus (entry_path);
}


void SubversionSetupDialog::on_entry_changed (GtkEditable *editable, gpointer user_data)
{
  ((SubversionSetupDialog *) user_data)->on_entry ();
}

  
void SubversionSetupDialog::on_entry ()
{
  if (event_id_gui) {
    GSource *source = g_main_context_find_source_by_id (NULL, event_id_gui);
    if (source) g_source_destroy (source);
  }
  event_id_gui = g_timeout_add_full (G_PRIORITY_DEFAULT, 600, GSourceFunc (delayer), gpointer(this), NULL);
}


bool SubversionSetupDialog::delayer (gpointer user_data)
{
  ((SubversionSetupDialog *) user_data)->delayed ();
  return false;
}


void SubversionSetupDialog::delayed ()
{
  set_gui ();
}


void SubversionSetupDialog::copy_project_to_repository ()
{
  // Repository path.
  ustring repositorypath = newrepository ();

  // Remove everything from the repository.
  {
    vector <ustring> repository_files;
    {
      GwSpawn spawn ("svn");
      spawn.arg ("ls");
      spawn.arg (repositorypath);
      spawn.arg ("--non-interactive");
      spawn.progress ("Retrieving repository contents", false);
      spawn.read ();
      spawn.run ();
      repository_files = spawn.standardout;
    }
    ProgressWindow progresswindow ("Erasing repository", false);
    progresswindow.set_iterate (0, 1, repository_files.size());
    for (unsigned int i = 0; i < repository_files.size(); i++) {
      progresswindow.iterate ();
      ustring path = gw_build_filename (repositorypath, repository_files[i]);
      GwSpawn spawn ("svn");
      spawn.arg ("rm");
      spawn.arg (path);
      spawn.arg ("--non-interactive");
      spawn.arg ("-m");
      spawn.arg ("Erase repository");
      spawn.run ();
    }    
  }

  // Directories.
  GeneralConfiguration genconfig (0);
  ustring project = genconfig.project();
  ustring workingdirectory = gw_build_filename (directories_get_projects(), project);

  // Import the project into the new remote repository.
  bool task_was_done;
  {
    GwSpawn spawn ("svn");
    spawn.workingdirectory (workingdirectory);
    spawn.arg ("import");
    spawn.arg ("data");
    spawn.arg (repositorypath);
    spawn.arg ("-m");
    spawn.arg ("Initial import project");
    spawn.arg ("--non-interactive");
    spawn.progress ("Processing", false);
    spawn.run ();
    task_was_done = (spawn.exitstatus == 0);
  }

  // Check the project out so our data changes into a working copy.
  // Save our data in case things go wrong.
  if (task_was_done ) {
    ustring datadirectory = project_data_directory_project (project);
    ustring checkoutdirectory = gw_build_filename (workingdirectory, gw_path_get_basename (repositorypath));

    GwSpawn spawn ("svn");
    spawn.workingdirectory (workingdirectory);
    spawn.arg ("checkout");
    spawn.arg (repositorypath);
    spawn.arg ("--non-interactive");
    spawn.progress ("Processing", false);
    spawn.run ();
    task_was_done = (spawn.exitstatus == 0);

    if (task_was_done) {
      unix_rmdir (datadirectory);
      unix_mv (checkoutdirectory, datadirectory);
    } else {
      unix_rmdir (checkoutdirectory);
    }
  }
}


void SubversionSetupDialog::copy_repository_to_project ()
{
  // We're going to change the project.
  projectchanged = true;

  // Repository path.
  ustring repositorypath = newrepository ();

  // Directories.
  GeneralConfiguration genconfig (0);
  ustring project = genconfig.project();
  ustring workingdirectory = gw_build_filename (directories_get_projects(), project);

  // Check the project out so our data changes into a working copy.
  // Save our data in case things go wrong.
  bool task_was_done;
  {
    ustring datadirectory = project_data_directory_project (project);
    ustring checkoutdirectory = gw_build_filename (workingdirectory, gw_path_get_basename (repositorypath));

    GwSpawn spawn ("svn");
    spawn.workingdirectory (workingdirectory);
    spawn.arg ("checkout");
    spawn.arg (repositorypath);
    spawn.arg ("--non-interactive");
    spawn.progress ("Processing", false);
    spawn.run ();
    task_was_done = (spawn.exitstatus == 0);

    if (task_was_done) {
      unix_rmdir (datadirectory);
      unix_mv (checkoutdirectory, datadirectory);
    } else {
      unix_rmdir (checkoutdirectory);
    }
  }
}


void SubversionSetupDialog::set_gui ()
{
  // Use remote repository?
  bool use_repository = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_external));

  // Set widgets sensitive depending on whether the repository is used.
  gtk_widget_set_sensitive (label_path, use_repository);
  gtk_widget_set_sensitive (entry_path, use_repository);
  gtk_widget_set_sensitive (labelusername, use_repository);
  gtk_widget_set_sensitive (entry_username, use_repository);
  gtk_widget_set_sensitive (labelpassword, use_repository);
  gtk_widget_set_sensitive (entry_password, use_repository);
  gtk_widget_set_sensitive (image_tasks_running_ok, use_repository);
  gtk_widget_set_sensitive (label_tasks_running_ok, use_repository);
  gtk_widget_set_sensitive (label_tasks_running_info, use_repository);
  gtk_widget_set_sensitive (image_repository_accessible_ok, use_repository);
  gtk_widget_set_sensitive (label_repository_accessible_done, use_repository);
  gtk_widget_set_sensitive (label_repository_accessible_info, use_repository);
  gtk_widget_set_sensitive (label_repository_accessible_output, use_repository);
  gtk_widget_set_sensitive (radiobutton2, use_repository);
  gtk_widget_set_sensitive (radiobutton3, use_repository);
  gtk_widget_set_sensitive (label53, use_repository);
  gtk_widget_set_sensitive (radiobutton_conflict_me, use_repository);
  gtk_widget_set_sensitive (radiobutton_conflict_server, use_repository);
  gtk_widget_set_sensitive (label54, use_repository);
  gtk_widget_set_sensitive (spinbutton_update, use_repository);
  gtk_widget_set_sensitive (label55, use_repository);

  // No repository used: set gui and bail out.
  if (!use_repository) {
    gui_okay (image_tasks_running_ok, label_tasks_running_ok, true);
    gtk_label_set_text (GTK_LABEL (label_tasks_running_info), "");
    gui_okay (image_repository_accessible_ok, label_repository_accessible_done, true);
    gtk_label_set_text (GTK_LABEL (label_repository_accessible_output), "");
    gtk_label_set_text (GTK_LABEL (label_repository_accessible_info), "");
    gtk_widget_set_sensitive (okbutton, true);
    return;   
  }

  // Set gui for the amount of tasks pending for the repository path.
  bool pending_tasks = project_tasks_count > 0;
  gui_okay (image_tasks_running_ok, label_tasks_running_ok, !pending_tasks);
  ustring tasks_pending_message;
  if (pending_tasks) {
    tasks_pending_message = "Waiting till the " + convert_to_string (project_tasks_count) + " pending tasks are done";
  } else {
    tasks_pending_message = "No pending tasks remaining";
  }
  gtk_label_set_text (GTK_LABEL (label_tasks_running_info), tasks_pending_message.c_str());

  // Pause processing subversion tasks if there are no pending tasks for this project.
  // This is because tasks will be produced, but these tasks are unwanted.
  // If for example a local book will be deleted, then it will be deleted 
  // from the remote repository too, which is unwanted behaviour.
  if (!pending_tasks) subversion_command_pause (true);
  
  // Get the repository path, username and password.
  ustring repositorypath = gtk_entry_get_text (GTK_ENTRY (entry_path));
  ustring username = gtk_entry_get_text (GTK_ENTRY (entry_username));
  ustring password = gtk_entry_get_text (GTK_ENTRY (entry_password));

  // Test write access to the repository.
  ustring write_access_elaborate_info;
  ustring authentication_test_filename = gw_build_filename (repositorypath, "bibledit_authentication_test");
  bool write_access = false;;
  if (!repositorypath.empty()) {
    GwSpawn spawn ("svn");
    if (!username.empty()) {
      spawn.arg ("--username");
      spawn.arg (username);
    }
    if (!password.empty()) {
      spawn.arg ("--password");
      spawn.arg (password);
    }
    spawn.arg ("--non-interactive");
    spawn.arg ("mkdir");
    spawn.arg (authentication_test_filename);
    spawn.arg ("-m");
    spawn.arg ("AuthenticationTest");    
    spawn.read ();
    spawn.run();
    for (unsigned int i = 0; i < spawn.standarderr.size(); i++) {
      if (i > 0) write_access_elaborate_info.append ("\n");
      write_access_elaborate_info.append (spawn.standarderr[i]);
    }
    write_access = spawn.exitstatus == 0;
  }
  if (!repositorypath.empty()) {
    GwSpawn spawn ("svn");
    if (!username.empty()) {
      spawn.arg ("--username");
      spawn.arg (username);
    }
    if (!password.empty()) {
      spawn.arg ("--password");
      spawn.arg (password);
    }
    spawn.arg ("--non-interactive");
    spawn.arg ("rm");
    spawn.arg (authentication_test_filename);
    spawn.arg ("-m");
    spawn.arg ("AuthenticationTest");    
    spawn.run();
  }
  // Repository ending on /data is not allowed. Would interfere with our own data directory.
  if (gw_path_get_basename (repositorypath) == "data")
    write_access = false;
  gui_okay (image_repository_accessible_ok, label_repository_accessible_done, write_access);
  ustring repository_accessible_info;
  if (write_access) {
    repository_accessible_info = "Write access has been granted";
    write_access_elaborate_info.clear();
  } else {
    repository_accessible_info = "Set correct values for repository path, username and password";
  }
  gtk_label_set_text (GTK_LABEL (label_repository_accessible_output), write_access_elaborate_info.c_str());
  gtk_label_set_text (GTK_LABEL (label_repository_accessible_info), repository_accessible_info.c_str());

  // Gui whether to allow action on the repository or local data.
  gtk_widget_set_sensitive (radiobutton2, write_access && repositorychanged ());
  gtk_widget_set_sensitive (radiobutton3, write_access && repositorychanged ());
  
  // Finally: whether to set okay button sensitive.
  bool ok_sensitive = true;
  if (use_repository) {
    if (repositorypath.empty()) ok_sensitive = false;
    if (!write_access) ok_sensitive = false;
  }
  if (!repositorychanged ()) ok_sensitive = true;
  if (repositorychanged () && pending_tasks) ok_sensitive = false;
  gtk_widget_set_sensitive (okbutton, ok_sensitive);
}


bool SubversionSetupDialog::tasks_counter_timeout (gpointer user_data)
{
  ((SubversionSetupDialog *) user_data)->tasks_counter ();
  return true;
}


void SubversionSetupDialog::tasks_counter ()
{
  GeneralConfiguration genconfig (0);
  project_tasks_count = subversion_count_tasks_project (genconfig.project());
  if (project_tasks_count != previous_project_tasks_count) {
    previous_project_tasks_count = project_tasks_count;
    set_gui ();
  }
}


void SubversionSetupDialog::on_okbutton_clicked (GtkButton *button, gpointer user_data)
{
  ((SubversionSetupDialog *) user_data)->on_okbutton ();
}


void SubversionSetupDialog::on_okbutton ()
{
  // Save variables.
  GeneralConfiguration genconfig (0);
  ustring project = genconfig.project ();
  ProjectConfiguration projectconfig (project);
  projectconfig.repository_username_set (gtk_entry_get_text (GTK_ENTRY (entry_username)));
  projectconfig.repository_password_set (gtk_entry_get_text (GTK_ENTRY (entry_password)));
  // Save conflict handling system.
  SubversionConflictHandlingType conflicthandling = schtTakeMe;
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_conflict_server)))
    conflicthandling = schtTakeServer;
  /*
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_conflict_prompt)))
    conflicthandling = schtPrompt;
  */
  projectconfig.repository_conflict_set (conflicthandling);
  // If repository changed, take actions set in the dialog.
  if (repositorychanged ()) {
    if (newrepository () == subversion_repository_local (project, true)) {
      // Repository returns to the default internal one.
      subversion_revert_to_internal_repository (project);
    } else {
      // Set in config that we use a remote repository.
      ProjectConfiguration projectconfig (project);
      projectconfig.repository_remote_set (true);
      // Take the right action.
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton2))) {
        copy_project_to_repository ();
      }
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton3))) {
        copy_repository_to_project ();
      }
    }
  }
  // Save update frequency.
  updatefrequency = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton_update));
  genconfig.subversion_update_frequency_set (updatefrequency);
}


ustring SubversionSetupDialog::newrepository ()
// Retrieves the new repository, from our settings.
{
  GeneralConfiguration genconfig (0);
  ustring repository = subversion_repository_local (genconfig.project(), true);
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radiobutton_external))) {
    repository = gtk_entry_get_text (GTK_ENTRY (entry_path));
  }
  return repository;
}


bool SubversionSetupDialog::repositorychanged ()
// Returns whether the repository is different from the one now used.
{
  GeneralConfiguration genconfig (0);
  return (newrepository () != subversion_project_get_url (genconfig.project()));
}
