/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogviewresources.h"
#include "utilities.h"
#include "generalconfig.h"
#include "help.h"
#include "resource_viewer.h"
#include "gtkwrappers.h"


ViewResourcesDialog::ViewResourcesDialog (int dummy):
  genconfig (0)
{
  viewresourcesdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (viewresourcesdialog), "Bibledit Resource Viewer");
  gtk_window_set_position (GTK_WINDOW (viewresourcesdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (viewresourcesdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (viewresourcesdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, FALSE, FALSE, 0);

  label12 = gtk_label_new ("Here you can set which resources will be loaded in the Bibledit Resource Viewer when Bibledit starts.\nYou can manually display resources too.");
  gtk_widget_show (label12);
  gtk_box_pack_start (GTK_BOX (vbox1), label12, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label12), 0, 0.5);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 0);

  // This part of the gui has been moved.

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox7, TRUE, TRUE, 0);

  button_add = gtk_button_new ();
  gtk_widget_show (button_add);
  gtk_box_pack_start (GTK_BOX (hbox7), button_add, FALSE, FALSE, 0);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (button_add), alignment2);

  hbox8 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox8);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox8);

  image2 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_box_pack_start (GTK_BOX (hbox8), image2, FALSE, FALSE, 0);

  label18 = gtk_label_new_with_mnemonic ("_Add resource");
  gtk_widget_show (label18);
  gtk_box_pack_start (GTK_BOX (hbox8), label18, FALSE, FALSE, 0);

  dialog_action_area1 = GTK_DIALOG (viewresourcesdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  help_button (dialog_action_area1, NULL, "view_resources");

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (viewresourcesdialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (viewresourcesdialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) button_add, "clicked",
                    G_CALLBACK (on_button_add_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (on_okbutton_clicked),
                    gpointer(this));

  gtk_widget_grab_default (okbutton);
  gtk_widget_grab_focus (okbutton);

  // Set the gui.
  vector<ustring> resources = genconfig.resources_viewer_urls ();
  for (unsigned int i = 0; i < resources.size(); i++) {
    on_button_add (resources[i]);
  }
}


ViewResourcesDialog::~ViewResourcesDialog ()
{
  gtk_widget_destroy (viewresourcesdialog);
}


int ViewResourcesDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (viewresourcesdialog));
}


void ViewResourcesDialog::on_button_open_clicked (GtkButton *button, gpointer user_data)
{
  ((ViewResourcesDialog *) user_data)->on_button_open (button);
}


void ViewResourcesDialog::on_button_open (GtkButton *button)
// Display the resource given in the entry.
{
  for (unsigned int i = 0; i < buttons_open.size(); i++) {
    if (GTK_WIDGET (button) == buttons_open[i]) {
      ustring url = gtk_entry_get_text (GTK_ENTRY (entries[i]));
      url = gtkw_file_chooser_open (viewresourcesdialog, "", url);
      if (!url.empty()) gtk_entry_set_text (GTK_ENTRY (entries[i]), url.c_str());      
    }
  }
}


void ViewResourcesDialog::on_button_display_clicked (GtkButton *button, gpointer user_data)
{
  ((ViewResourcesDialog *) user_data)->on_button_display (button);
}


void ViewResourcesDialog::on_button_display (GtkButton *button)
// Display the resource given in the entry.
{
  for (unsigned int i = 0; i < buttons_display.size(); i++) {
    if (GTK_WIDGET (button) == buttons_display[i]) {
      ustring url = gtk_entry_get_text (GTK_ENTRY (entries[i]));
      if (!url.empty()) {
        resource_viewer_open_page (url);
      }
    }
  }
}


void ViewResourcesDialog::on_button_remove_clicked (GtkButton *button, gpointer user_data)
{
  ((ViewResourcesDialog *) user_data)->on_button_remove (button);
}


void ViewResourcesDialog::on_button_remove (GtkButton *button)
// Removes a resource.
{
  // Get the offset of the widget to remove.
  int widget_offset = -1;
  for (unsigned int i = 0; i < buttons_remove.size(); i++) {
    if (GTK_WIDGET (button) == buttons_remove[i])
      widget_offset = i;
  }
  // Widget not found: bail out.
  if (widget_offset < 0)
    return;
  // Destroy the widgets.
  gtk_widget_destroy (hboxes[widget_offset]);
  // Remove widget's pointers.
  vector_remove_content (hboxes, widget_offset);
  vector_remove_content (labels, widget_offset);
  vector_remove_content (entries, widget_offset);
  vector_remove_content (buttons_open, widget_offset);
  vector_remove_content (buttons_display, widget_offset);
  vector_remove_content (buttons_remove, widget_offset);
  // Rewrite the labels.
  rewrite_button_labels ();
}


void ViewResourcesDialog::on_button_add_clicked (GtkButton *button, gpointer user_data)
{
  ((ViewResourcesDialog *) user_data)->on_button_add ("");
}


void ViewResourcesDialog::on_button_add (const ustring& url)
// Adds a new project to the gui.
{
  // All the widgets we're going to insert.  
  GtkWidget *hbox5;
  GtkWidget *label_resource;
  GtkWidget *entry1;
  GtkWidget *button_open;
  GtkWidget *alignment4;
  GtkWidget *hbox12;
  GtkWidget *image4;
  GtkWidget *label21;
  GtkWidget *button_display;
  GtkWidget *alignment3;
  GtkWidget *hbox11;
  GtkWidget *image3;
  GtkWidget *label20;
  GtkWidget *button_remove;
  GtkWidget *alignment1;
  GtkWidget *hbox6;
  GtkWidget *image1;
  GtkWidget *label17;
  
  // Build GUI.
  hbox5 = gtk_hbox_new (FALSE, 4);
  gtk_widget_show (hbox5);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox5, TRUE, TRUE, 0);

  label_resource = gtk_label_new_with_mnemonic ("");
  gtk_widget_show (label_resource);
  gtk_box_pack_start (GTK_BOX (hbox5), label_resource, FALSE, FALSE, 0);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox5), entry1, TRUE, TRUE, 0);
  gtk_entry_set_width_chars (GTK_ENTRY (entry1), 70);

  // Set the resource in the entry.
  gtk_entry_set_text (GTK_ENTRY (entry1), url.c_str());

  button_open = gtk_button_new ();
  gtk_widget_show (button_open);
  gtk_box_pack_start (GTK_BOX (hbox5), button_open, FALSE, FALSE, 0);

  alignment4 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment4);
  gtk_container_add (GTK_CONTAINER (button_open), alignment4);

  hbox12 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox12);
  gtk_container_add (GTK_CONTAINER (alignment4), hbox12);

  image4 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image4);
  gtk_box_pack_start (GTK_BOX (hbox12), image4, FALSE, FALSE, 0);

  label21 = gtk_label_new_with_mnemonic ("Open");
  gtk_widget_show (label21);
  gtk_box_pack_start (GTK_BOX (hbox12), label21, FALSE, FALSE, 0);

  button_display = gtk_button_new ();
  gtk_widget_show (button_display);
  gtk_box_pack_start (GTK_BOX (hbox5), button_display, FALSE, FALSE, 0);

  alignment3 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment3);
  gtk_container_add (GTK_CONTAINER (button_display), alignment3);

  hbox11 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox11);
  gtk_container_add (GTK_CONTAINER (alignment3), hbox11);

  image3 = gtk_image_new_from_stock ("gtk-justify-fill", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image3);
  gtk_box_pack_start (GTK_BOX (hbox11), image3, FALSE, FALSE, 0);

  label20 = gtk_label_new_with_mnemonic ("Display");
  gtk_widget_show (label20);
  gtk_box_pack_start (GTK_BOX (hbox11), label20, FALSE, FALSE, 0);

  button_remove = gtk_button_new ();
  gtk_widget_show (button_remove);
  gtk_box_pack_start (GTK_BOX (hbox5), button_remove, FALSE, FALSE, 0);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (button_remove), alignment1);

  hbox6 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox6);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox6);

  image1 = gtk_image_new_from_stock ("gtk-remove", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox6), image1, FALSE, FALSE, 0);

  label17 = gtk_label_new ("Remove");
  gtk_widget_show (label17);
  gtk_box_pack_start (GTK_BOX (hbox6), label17, FALSE, FALSE, 0);

  g_signal_connect ((gpointer) button_open, "clicked",
                    G_CALLBACK (on_button_open_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) button_display, "clicked",
                    G_CALLBACK (on_button_display_clicked),
                    gpointer(this));
  g_signal_connect ((gpointer) button_remove, "clicked",
                    G_CALLBACK (on_button_remove_clicked),
                    gpointer(this));
                    
  gtk_label_set_mnemonic_widget (GTK_LABEL (label_resource), entry1);

  gtk_box_reorder_child (GTK_BOX (vbox1), hbox5, hboxes.size() + 2);

  // Store the relevant widgets.
  hboxes.push_back (hbox5);
  labels.push_back (label_resource);
  entries.push_back (entry1);
  buttons_open.push_back (button_open);
  buttons_display.push_back (button_display);
  buttons_remove.push_back (button_remove);
  
  // Reorder the accelerators.
  rewrite_button_labels ();
}


void ViewResourcesDialog::on_okbutton_clicked (GtkButton *button, gpointer user_data)
{
  ((ViewResourcesDialog *) user_data)->on_okbutton ();
}


void ViewResourcesDialog::on_okbutton ()
{
  // Save resources.
  vector<ustring> resources;
  for (unsigned int i = 0; i < entries.size(); i++) {
    ustring resource = gtk_entry_get_text (GTK_ENTRY (entries[i]));
    if (!resource.empty()) {
      resources.push_back (resource);
    }
  }
  genconfig.resources_viewer_urls_set (resources);
}


void ViewResourcesDialog::vector_remove_content (vector<GtkWidget *>& container, int offset)
{
  vector<GtkWidget *>::iterator iter;
  iter = container.begin();
  for (int i = 0; i < offset; i++)
    iter++;
  container.erase (iter);    
}


void ViewResourcesDialog::rewrite_button_labels ()
// Rewrites the accelerators on the buttons so that it goes from 1 till x.
{
  for (unsigned int i = 0; i < entries.size(); i++) {
    ustring label = "_" + convert_to_string (i + 1);
    gtk_label_set_text_with_mnemonic (GTK_LABEL (labels[i]), label.c_str());
  }
}
