/*
** Copyright (C) 2003-2006 Teus Benschop.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef INCLUDED_SUBVERSION_H
#define INCLUDED_SUBVERSION_H


#include "libraries.h"
#include <glib.h>
#include <gtk/gtk.h>
#include "categorize.h"
#include "reference.h"
#include "types.h"


// Possible results of running svn update.
enum SubversionUpdateType {sutOk, sutUpdated, sutAdded, sutDeleted, sutReplaced, sutMerged, sutConflict, sutError};
// Conflict handling.
enum SubversionConflictHandlingType {schtTakeMe, schtTakeServer};


ustring subversion_repository_local (const ustring& project, bool includeprefix);
void subversion_initial_check_all (bool gui);
void subversion_initial_check_project (const ustring& project);
void subversion_store_chapter (const ustring& project, unsigned int book, unsigned int chapter);
void subversion_move_project (const ustring& project, const ustring& newproject);
void subversion_remove_book (const ustring& project, unsigned int book);
void subversion_remove_chapter (const ustring& project, unsigned int book, unsigned int chapter);
void subversion_remove_project (const ustring& project);
void subversion_vacuum ();
void subversion_cleanup ();
void subversion_get_tasks (vector <unsigned int>& tasks, vector <ustring>& projects,
                           vector <unsigned int>& books, vector <unsigned int>& chapters,
                           vector <unsigned int>& fails);
void subversion_task_human_readable (unsigned int task, const ustring& project,
                                     unsigned int book, unsigned int chapter,
                                     unsigned int fail,
                                     gchar * & human_readable_task,
                                     ustring& human_readable_description,
                                     ustring& human_readable_status);
void subversion_erase_tasks_project (const ustring& project);
int subversion_count_tasks_project (const ustring& project);
void subversion_open_chapter_for_editing (const ustring& project, unsigned int book, unsigned int chapter);
bool subversion_execute_update_project (const ustring& project);
SubversionUpdateType subversion_update_chapter_remote (const ustring& project, unsigned int book, unsigned int chapter, bool& ran_okay);
void subversion_update_chapter_remote_scheduler (const ustring& project, unsigned int book, unsigned int chapter, SubversionUpdateType updatetype, Reference * editor_reference);
ustring subversion_project_get_url (const ustring& project);
void subversion_revert_to_internal_repository (const ustring& project);
void subversion_resolve_conflict_chapter (const ustring& project, unsigned int book, unsigned int chapter);
unsigned int subversion_log_date_time_at_revision (const ustring& project, unsigned int revision);
ustring subversion_revision_date_time (unsigned int seconds);
void subversion_get_chapters_changed_since (const ustring& project, int seconds, vector <unsigned int>& books, vector <unsigned int>& chapters);
void subversion_log_get_seconds_and_revisions (const vector<ustring>& log, vector <unsigned int>& seconds, vector <unsigned int>& revisions);
ustring subversion_command_database ();
void subversion_command_initialize_subsystem ();
void subversion_command_finalize_subsystem ();
bool subversion_execute_get_next_task (unsigned int& task, ustring& project, unsigned int& book, unsigned int& chapter);
void subversion_execute_erase_task (unsigned int task, const ustring& project, unsigned int book, unsigned int chapter);
void subversion_execute_record_failure (unsigned int task, const ustring& project, unsigned int book, unsigned int chapter);
unsigned int subversion_command_scheduled_tasks_count ();
bool subversion_execute_store_chapter (const ustring& project, unsigned int book, unsigned int chapter);
void subversion_command_store_reference (const Reference& reference);
Reference subversion_execute_retrieve_reference ();
bool subversion_execute_initialize_project_local (const ustring& project);
bool subversion_execute_commit_project (const ustring& project);
void subversion_store_re_open_project (const ustring& project);
bool subversion_retrieve_re_open_project ();
void subversion_schedule (SubversionTaskType task, const ustring& project, unsigned int book, unsigned int chapter);
void subversion_command_pause (bool pause);
void subversion_command_set_update_frequency (unsigned int frequency);
ustring subversion_mine_conflict_marker ();


#endif
